import josx.platform.rcx.*;

public class CircuitRobot extends Robot {
	// Zustand des Roboter fuer setPower-Methode
	private Balance balance;

	// Helligkeitsstufen der beiden Sensoren
	private byte[] brightLight;
	private byte LightTolerance = 11;
	private int rotationTime = 220; 

	// Sensoren-Konstanten zu Erhoehung
	// der Lesbarkeit des Quellcodes
	private final byte S1 = 0;
	private final byte S3 = 2;
	
	public CircuitRobot() {
		// Waage instanziieren
		balance = new Balance(this); 
		balance.setStopHandling();
		balance.start();
		
		// Lichttoleranzwert bzw. Rotationszeit spezifisch
		// fuer die Strecke bzw. fr den Batteriestand anpassen
		setLightTolerance();
		setRotationTime();
		
		// Lichtensoren initalisieren
		initSensor(S1);
		initSensor(S3);
		
		// PRGM durch den ButtonListener Robot
		// ueberwachen lassen, damit der Roboter
		// durch Druecken von PRGM "stirbt"
		Button.PRGM.addButtonListener(this);
		
		// normales helles Licht ermitteln
		setNormalBrightLight();
		
		// onS1 und onS2 sind wahr, wenn der
		// jeweilige Sensor auf der Linie ist
		boolean onS1, onS3;
		
		// isDriving ist wahr, wenn der Roboter geradeaus fdhrt
		boolean isDriving = false;

		// fahren, solange der Roboter "lebt"
		while (isAlive()) {
			onS1 = isOnLine(S1);
			onS3 = isOnLine(S3);

			// losfahren, wenn der Roboter nicht faehrt
			// und beide Sensoren nicht auf der Linie sind
			if (!isDriving && !(onS1 && onS3)) {
				balance.forward();
				move(CIRCUIT_FORWARD);
				isDriving = true;
			}
			
			// drehen, wenn der Roboter faehrt und wenn
			// ein mindestens Sensor auf der Linie ist 
			if (isDriving && (onS1 || onS3)) {
				rotateUntilOffLine(onS1, onS3);
				isDriving = false;
			}
		}

		// Roboter "sterben" lassen
		move(FLOAT);
		Sensor.S1.passivate();
		Sensor.S3.passivate();
	}
	
	// Systemzeit als int-Wert zur|ckgeben
	private int getTime() {
		return (int) System.currentTimeMillis();
	}
	
	// der Roboter dreht solange, bis er nicht mehr auf der Linie ist
	private void rotateUntilOffLine(boolean onS1, boolean onS3) {
		boolean left = onS1 && !onS3;
		boolean right = !onS1 && onS3;
		
		if (left || right) {
			byte fstSensor = left ? S1 : S3; 
			byte sndSensor = left ? S3 : S1;

			// solange drehen bis Sensor nicht mehr auf der Linie  
			if (left) {
				balance.left();
				move(CIRCUIT_LEFT);
			} else {
				balance.right();
				move(CIRCUIT_RIGHT);
			}
			while (isOnLine(fstSensor) && isAlive());

			// wenn Roboter waagerecht, bis zur Mitte weiterdrehen
			if (balance.isHorizontal) {
				int time = getTime() + rotationTime;
				while (time >= getTime() && !isOnLine(sndSensor) && balance.isHorizontal && isAlive());
			}
		}
	}

	// Methode zum Pruefen, ob ein Sensor auf der Linie ist
	private boolean isOnLine(int sensor) {
		return Sensor.SENSORS[sensor].readValue() < brightLight[sensor];
	}

	// Methode zum Initialisieren der Sensoren
	private void initSensor(byte sensor) {
		Sensor.SENSORS[sensor].setTypeAndMode(
				SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_PCT);
		Sensor.SENSORS[sensor].setPreviousValue(0);
		Sensor.SENSORS[sensor].activate();
	}

	// normales helles Licht ermitteln
	private void setNormalBrightLight() {
		int sumS1 = 0, sumS3 = 0, dataCount = 1000;
		for (int i = 0; i < dataCount; i++) {
			sumS1 += Sensor.S1.readValue();
			sumS3 += Sensor.S3.readValue();
		}
		brightLight = new byte[3];
		brightLight[S1] = (byte) (sumS1 / dataCount - LightTolerance);
		brightLight[S3] = (byte) (sumS3 / dataCount - LightTolerance);
		LCD.setSegment(LCDConstants.WALKING);
		LCD.refresh();
	}
	
	// Methode zum Setzen des Toleranzwerts (1 bis 20)
	// Wert mit VIEW einstellen und mit PRGM abschliessen  
	private void setLightTolerance() {
		LCD.showNumber(LightTolerance);
		balance.showPosition();
		
		while (!Button.PRGM.isPressed()) {
			if (Button.VIEW.isPressed()) {
				LightTolerance = (byte) (LightTolerance % 20 + 1);
				LCD.showNumber(LightTolerance);
				pause(200);
			}
		}
		while (Button.PRGM.isPressed());
		
		LCD.clear();
		LCD.refresh();
		Sound.twoBeeps();
		balance.showPosition();
	}

	// Methode zum Setzen der Rotationszeit in msec (100 bis 300)
	// Wert mit VIEW einstellen und mit PRGM abschliessen  
	private void setRotationTime() {
		LCD.showNumber(rotationTime);
		balance.showPosition();
		
		while (!Button.PRGM.isPressed()) {
			if (Button.VIEW.isPressed()) {
				rotationTime += 10;
				if (rotationTime == 310) rotationTime = 100;
				LCD.showNumber(rotationTime);
				pause(200);
			}
		}
		while (Button.PRGM.isPressed());
		
		LCD.clear();
		LCD.refresh();
		Sound.twoBeeps();
		balance.showPosition();
	}

	public static void main(String[] args) {
		new CircuitRobot();
	}
}